@@ -55,7 +55,6 @@
 #include "empathy-cell-renderer-activatable.h"
 #include "empathy-ui-utils.h"
 #include "empathy-gtk-enum-types.h"
-#include "empathy-gtk-marshal.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
@@ -75,6 +74,7 @@ typedef struct
 
   gboolean show_offline;
   gboolean show_untrusted;
+  gboolean show_uninteresting;
 
   GtkTreeModelFilter *filter;
   GtkWidget *search_widget;
@@ -115,6 +115,7 @@ enum
   PROP_INDIVIDUAL_FEATURES,
   PROP_SHOW_OFFLINE,
   PROP_SHOW_UNTRUSTED,
+  PROP_SHOW_UNINTERESTING,
 };
 
 /* TODO: re-add DRAG_TYPE_CONTACT_ID, for the case that we're dragging around
@@ -132,8 +133,8 @@ typedef enum
   { (gchar *) T, 0, I }
 
 static const GtkTargetEntry drag_types_dest[] = {
-  DRAG_TYPE ("text/individual-id", DND_DRAG_TYPE_INDIVIDUAL_ID),
-  DRAG_TYPE ("text/persona-id", DND_DRAG_TYPE_PERSONA_ID),
+  DRAG_TYPE ("text/x-individual-id", DND_DRAG_TYPE_INDIVIDUAL_ID),
+  DRAG_TYPE ("text/x-persona-id", DND_DRAG_TYPE_PERSONA_ID),
   DRAG_TYPE ("text/path-list", DND_DRAG_TYPE_URI_LIST),
   DRAG_TYPE ("text/uri-list", DND_DRAG_TYPE_URI_LIST),
   DRAG_TYPE ("text/plain", DND_DRAG_TYPE_STRING),
@@ -141,7 +142,7 @@ static const GtkTargetEntry drag_types_dest[] = {
 };
 
 static const GtkTargetEntry drag_types_source[] = {
-  DRAG_TYPE ("text/individual-id", DND_DRAG_TYPE_INDIVIDUAL_ID),
+  DRAG_TYPE ("text/x-individual-id", DND_DRAG_TYPE_INDIVIDUAL_ID),
 };
 
 #undef DRAG_TYPE
@@ -866,7 +867,7 @@ individual_view_drag_data_get (GtkWidget *widget,
   if (info == DND_DRAG_TYPE_INDIVIDUAL_ID)
     {
       gtk_selection_data_set (selection,
-          gdk_atom_intern ("text/individual-id", FALSE), 8,
+          gdk_atom_intern ("text/x-individual-id", FALSE), 8,
           (guchar *) individual_id, strlen (individual_id) + 1);
     }
 
@@ -1533,7 +1534,7 @@ expand_idle_foreach_cb (GtkTreeModel *model,
       EMPATHY_INDIVIDUAL_STORE_COL_NAME, &name,
       -1);
 
-  if (is_group == FALSE)
+  if (!is_group)
     {
       g_free (name);
       return FALSE;
@@ -1542,9 +1543,9 @@ expand_idle_foreach_cb (GtkTreeModel *model,
   priv = GET_PRIV (self);
 
   if (g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
-      &should_expand) == TRUE)
+      &should_expand))
     {
-      if (GPOINTER_TO_INT (should_expand) == TRUE)
+      if (GPOINTER_TO_INT (should_expand))
         gtk_tree_view_expand_row (GTK_TREE_VIEW (self), path, FALSE);
       else
         gtk_tree_view_collapse_row (GTK_TREE_VIEW (self), path);
@@ -1623,8 +1624,8 @@ individual_view_row_has_child_toggled_cb (GtkTreeModel *model,
    * gtk_tree_model_filter_refilter (). We add the rows to expand/contract to
    * a hash table, and expand or contract them as appropriate all at once in
    * an idle handler which iterates over all the group rows. */
-  if (g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
-      &will_expand) == FALSE ||
+  if (!g_hash_table_lookup_extended (priv->expand_groups, name, NULL,
+      &will_expand) ||
       GPOINTER_TO_INT (will_expand) != should_expand)
     {
       g_hash_table_insert (priv->expand_groups, g_strdup (name),
@@ -1702,7 +1703,7 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
   EmpathyLiveSearch *live = EMPATHY_LIVE_SEARCH (priv->search_widget);
   GeeSet *personas;
   GeeIterator *iter;
-  gboolean is_favorite, contains_interesting_persona = FALSE;
+  gboolean is_favorite;
 
   /* Always display individuals having pending events */
   if (event_count > 0)
@@ -1710,32 +1711,38 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
 
   /* We're only giving the visibility wrt filtering here, not things like
    * presence. */
-  if (priv->show_untrusted == FALSE &&
+  if (!priv->show_untrusted &&
       folks_individual_get_trust_level (individual) == FOLKS_TRUST_LEVEL_NONE)
     {
       return FALSE;
     }
 
-  /* Hide all individuals which consist entirely of uninteresting personas */
-  personas = folks_individual_get_personas (individual);
-  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
-  while (!contains_interesting_persona && gee_iterator_next (iter))
+  if (!priv->show_uninteresting)
     {
-      FolksPersona *persona = gee_iterator_get (iter);
+      gboolean contains_interesting_persona = FALSE;
 
-      if (empathy_folks_persona_is_interesting (persona))
-        contains_interesting_persona = TRUE;
+      /* Hide all individuals which consist entirely of uninteresting
+       * personas */
+      personas = folks_individual_get_personas (individual);
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (!contains_interesting_persona && gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
 
-      g_clear_object (&persona);
+          if (empathy_folks_persona_is_interesting (persona))
+            contains_interesting_persona = TRUE;
+
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
+
+      if (!contains_interesting_persona)
+        return FALSE;
     }
-  g_clear_object (&iter);
-
-  if (contains_interesting_persona == FALSE)
-    return FALSE;
 
   is_favorite = folks_favourite_details_get_is_favourite (
       FOLKS_FAVOURITE_DETAILS (individual));
-  if (is_searching == FALSE) {
+  if (!is_searching) {
     if (is_favorite && is_fake_group &&
         !tp_strdiff (group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
         /* Always display favorite contacts in the favorite group */
@@ -1814,7 +1821,7 @@ individual_view_filter_visible_func (GtkTreeModel *model,
       g_free (group);
 
       /* FIXME: Work around bgo#626552/bgo#621076 */
-      if (visible == TRUE)
+      if (visible)
         {
           GtkTreePath *path = gtk_tree_model_get_path (model, iter);
           individual_view_verify_group_visibility (self, path);
@@ -1855,7 +1862,7 @@ individual_view_filter_visible_func (GtkTreeModel *model,
       g_free (group);
 
       /* show group if it has at least one visible contact in it */
-      if (visible == TRUE)
+      if (visible)
         return TRUE;
     }
 
@@ -2047,7 +2054,7 @@ individual_view_finalize (GObject *object)
 
   if (priv->expand_groups_idle_handler != 0)
     g_source_remove (priv->expand_groups_idle_handler);
-  g_hash_table_destroy (priv->expand_groups);
+  g_hash_table_unref (priv->expand_groups);
 
   G_OBJECT_CLASS (empathy_individual_view_parent_class)->finalize (object);
 }
@@ -2079,6 +2086,9 @@ individual_view_get_property (GObject *object,
     case PROP_SHOW_UNTRUSTED:
       g_value_set_boolean (value, priv->show_untrusted);
       break;
+    case PROP_SHOW_UNINTERESTING:
+      g_value_set_boolean (value, priv->show_uninteresting);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -2113,6 +2123,9 @@ individual_view_set_property (GObject *object,
       empathy_individual_view_set_show_untrusted (view,
           g_value_get_boolean (value));
       break;
+    case PROP_SHOW_UNINTERESTING:
+      empathy_individual_view_set_show_uninteresting (view,
+          g_value_get_boolean (value));
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -2152,7 +2165,7 @@ empathy_individual_view_class_init (EmpathyIndividualViewClass *klass)
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (EmpathyIndividualViewClass, drag_individual_received),
       NULL, NULL,
-      _empathy_gtk_marshal_VOID__UINT_OBJECT_STRING_STRING,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE, 4, G_TYPE_UINT, FOLKS_TYPE_INDIVIDUAL,
       G_TYPE_STRING, G_TYPE_STRING);
 
@@ -2162,7 +2175,7 @@ empathy_individual_view_class_init (EmpathyIndividualViewClass *klass)
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (EmpathyIndividualViewClass, drag_persona_received),
       NULL, NULL,
-      _empathy_gtk_marshal_BOOLEAN__UINT_OBJECT_OBJECT,
+      g_cclosure_marshal_generic,
       G_TYPE_BOOLEAN, 3, G_TYPE_UINT, FOLKS_TYPE_PERSONA, FOLKS_TYPE_INDIVIDUAL);
 
   g_object_class_install_property (object_class,
@@ -2199,6 +2212,13 @@ empathy_individual_view_class_init (EmpathyIndividualViewClass *klass)
           "Whether the view should display untrusted individuals; "
           "those who could not be who they say they are.",
           TRUE, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+      PROP_SHOW_UNINTERESTING,
+      g_param_spec_boolean ("show-uninteresting",
+          "Show Uninteresting Individuals",
+          "Whether the view should not filter out individuals using "
+          "empathy_folks_persona_is_interesting.",
+          FALSE, G_PARAM_READWRITE));
 
   g_type_class_add_private (object_class, sizeof (EmpathyIndividualViewPriv));
 }
@@ -2212,6 +2232,7 @@ empathy_individual_view_init (EmpathyIndividualView *view)
   view->priv = priv;
 
   priv->show_untrusted = TRUE;
+  priv->show_uninteresting = FALSE;
 
   /* Get saved group states. */
   empathy_contact_groups_get_all ();
@@ -2453,12 +2474,12 @@ got_avatar (GObject *source_object,
 {
   FolksIndividual *individual = FOLKS_INDIVIDUAL (source_object);
   EmpathyIndividualView *view = user_data;
+  EmpathyIndividualViewPriv *priv = GET_PRIV (view);
   GdkPixbuf *avatar;
   EmpathyIndividualManager *manager;
   gchar *text;
   GtkWindow *parent;
   GeeSet *personas;
-  GeeIterator *iter;
   guint persona_count = 0;
   gboolean can_block;
   GError *error = NULL;
@@ -2477,21 +2498,31 @@ got_avatar (GObject *source_object,
    * so we still display the remove dialog. */
 
   personas = folks_individual_get_personas (individual);
-  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+
+  if (priv->show_uninteresting)
+    {
+      persona_count = gee_collection_get_size (GEE_COLLECTION (personas));
+    }
+  else
+    {
+      GeeIterator *iter;
+
+      iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+      while (persona_count < 2 && gee_iterator_next (iter))
+        {
+          FolksPersona *persona = gee_iterator_get (iter);
+
+          if (empathy_folks_persona_is_interesting (persona))
+            persona_count++;
+
+          g_clear_object (&persona);
+        }
+      g_clear_object (&iter);
+    }
 
   /* If we have more than one TpfPersona, display a different message
    * ensuring the user knows that *all* of the meta-contacts' personas will
    * be removed. */
-  while (persona_count < 2 && gee_iterator_next (iter))
-    {
-      FolksPersona *persona = gee_iterator_get (iter);
-
-      if (empathy_folks_persona_is_interesting (persona))
-        persona_count++;
-
-      g_clear_object (&persona);
-    }
-  g_clear_object (&iter);
 
   if (persona_count < 2)
     {
@@ -2596,6 +2627,9 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   if (individual == NULL)
     return NULL;
 
+  if (!empathy_folks_individual_contains_contact (individual))
+    goto out;
+
   /* If any of the Individual's personas can be removed, add an option to
    * remove. This will act as a best-effort option. If any Personas cannot be
    * removed from the server, then this option will just be inactive upon
@@ -2650,6 +2684,7 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   g_signal_connect (menu, "link-contacts-activated",
       (GCallback) individual_menu_link_contacts_activated_cb, view);
 
+out:
   g_object_unref (individual);
 
   return menu;
@@ -2862,3 +2897,36 @@ empathy_individual_view_refilter (EmpathyIndividualView *self)
 
   gtk_tree_model_filter_refilter (priv->filter);
 }
+
+void
+empathy_individual_view_select_first (EmpathyIndividualView *self)
+{
+  EmpathyIndividualViewPriv *priv = GET_PRIV (self);
+  GtkTreeIter iter;
+
+  gtk_tree_model_filter_refilter (priv->filter);
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->filter), &iter))
+    {
+      GtkTreeSelection *selection = gtk_tree_view_get_selection (
+          GTK_TREE_VIEW (self));
+
+      gtk_tree_selection_select_iter (selection, &iter);
+    }
+}
+
+void
+empathy_individual_view_set_show_uninteresting (EmpathyIndividualView *self,
+    gboolean show_uninteresting)
+{
+  EmpathyIndividualViewPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (self));
+
+  priv = GET_PRIV (self);
+
+  priv->show_uninteresting = show_uninteresting;
+
+  g_object_notify (G_OBJECT (self), "show-uninteresting");
+  gtk_tree_model_filter_refilter (priv->filter);
+}
