@@ -365,7 +365,7 @@ protocol_chooser_finalize (GObject *object)
 
   if (priv->protocols)
     {
-      g_hash_table_destroy (priv->protocols);
+      g_hash_table_unref (priv->protocols);
       priv->protocols = NULL;
     }
 
@@ -600,7 +600,10 @@ empathy_protocol_chooser_create_account_settings (EmpathyProtocolChooser *self)
 
       empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
           NULL, NULL);
-
+      empathy_account_settings_set_string (settings, "server",
+          extra_certificate_identities[0]);
+      empathy_account_settings_set_boolean (settings, "require-encryption",
+          TRUE);
       empathy_account_settings_set_strv (settings, "fallback-servers",
           fallback_servers);
 
@@ -615,7 +618,8 @@ empathy_protocol_chooser_create_account_settings (EmpathyProtocolChooser *self)
     {
       empathy_account_settings_set_icon_name_async (settings, "im-facebook",
           NULL, NULL);
-
+      empathy_account_settings_set_boolean (settings, "require-encryption",
+          TRUE);
       empathy_account_settings_set_string (settings, "server",
           "chat.facebook.com");
     }
