@@ -782,7 +782,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyContact        *sender;
 	TpMessage             *tp_msg;
 	TpAccount             *account;
-	gchar                 *body_escaped;
+	gchar                 *body_escaped, *name_escaped;
 	const gchar           *name;
 	const gchar           *contact_id;
 	EmpathyAvatar         *avatar;
@@ -816,6 +816,8 @@ theme_adium_append_message (EmpathyChatView *view,
 	contact_id = empathy_contact_get_id (sender);
 	action = (empathy_message_get_tptype (msg) == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION);
 
+	name_escaped = g_markup_escape_text (name, -1);
+
 	/* If this is a /me probably */
 	if (action) {
 		gchar *str;
@@ -823,7 +825,7 @@ theme_adium_append_message (EmpathyChatView *view,
 		if (priv->data->version >= 4 || !priv->data->custom_template) {
 			str = g_strdup_printf ("<span class='actionMessageUserName'>%s</span>"
 					       "<span class='actionMessageBody'>%s</span>",
-					       name, body_escaped);
+					       name_escaped, body_escaped);
 		} else {
 			str = g_strdup_printf ("*%s*", body_escaped);
 		}
@@ -948,7 +950,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	}
 
 	theme_adium_append_html (theme, func, html, body_escaped,
-				 avatar_filename, name, contact_id,
+				 avatar_filename, name_escaped, contact_id,
 				 service_name, message_classes->str,
 				 timestamp, is_backlog, empathy_contact_is_user (sender));
 
@@ -961,6 +963,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	priv->last_is_backlog = is_backlog;
 
 	g_free (body_escaped);
+	g_free (name_escaped);
 	g_string_free (message_classes, TRUE);
 }
 
@@ -981,6 +984,14 @@ theme_adium_append_event (EmpathyChatView *view,
 	g_free (str_escaped);
 }
 
+static void
+theme_adium_append_event_markup (EmpathyChatView *view,
+				 const gchar     *markup_text,
+				 const gchar     *fallback_text)
+{
+	theme_adium_append_event_escaped (view, markup_text);
+}
+
 static void
 theme_adium_edit_message (EmpathyChatView *view,
 			  EmpathyMessage  *message)
@@ -1297,6 +1308,7 @@ theme_adium_iface_init (EmpathyChatViewIface *iface)
 {
 	iface->append_message = theme_adium_append_message;
 	iface->append_event = theme_adium_append_event;
+	iface->append_event_markup = theme_adium_append_event_markup;
 	iface->edit_message = theme_adium_edit_message;
 	iface->scroll = theme_adium_scroll;
 	iface->scroll_down = theme_adium_scroll_down;
@@ -1654,6 +1666,20 @@ empathy_theme_adium_set_variant (EmpathyThemeAdium *theme,
 	g_object_notify (G_OBJECT (theme), "variant");
 }
 
+void
+empathy_theme_adium_show_inspector (EmpathyThemeAdium *theme)
+{
+	WebKitWebView      *web_view = WEBKIT_WEB_VIEW (theme);
+	WebKitWebInspector *inspector;
+
+	g_object_set (G_OBJECT (webkit_web_view_get_settings (web_view)),
+		      "enable-developer-extras", TRUE,
+		      NULL);
+
+	inspector = webkit_web_view_get_inspector (web_view);
+	webkit_web_inspector_show (inspector);
+}
+
 gboolean
 empathy_adium_path_is_valid (const gchar *path)
 {
