@@ -29,6 +29,7 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyAuthFactory EmpathyAuthFactory;
 typedef struct _EmpathyAuthFactoryClass EmpathyAuthFactoryClass;
+typedef struct _EmpathyAuthFactoryPriv EmpathyAuthFactoryPriv;
 
 struct _EmpathyAuthFactoryClass {
     TpBaseClientClass parent_class;
@@ -36,7 +37,7 @@ struct _EmpathyAuthFactoryClass {
 
 struct _EmpathyAuthFactory {
     TpBaseClient parent;
-    gpointer priv;
+    EmpathyAuthFactoryPriv *priv;
 };
 
 GType empathy_auth_factory_get_type (void);
@@ -58,7 +59,7 @@ GType empathy_auth_factory_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), EMPATHY_TYPE_AUTH_FACTORY, \
     EmpathyAuthFactoryClass))
 
-EmpathyAuthFactory * empathy_auth_factory_dup_singleton (void);
+EmpathyAuthFactory * empathy_auth_factory_new (TpSimpleClientFactory *factory);
 
 gboolean empathy_auth_factory_register (EmpathyAuthFactory *self,
     GError **error);
