@@ -115,6 +115,9 @@ typedef struct
   /* Service (CM, Client) chooser store */
   GtkListStore *service_store;
 
+  /* Debug to show upon creation */
+  gchar *select_name;
+
   /* Misc. */
   gboolean dispose_run;
   TpAccountManager *am;
@@ -628,7 +631,7 @@ get_cm_display_name (EmpathyDebugWindow *self,
       retval = g_strdup (cm_name);
     }
 
-  g_hash_table_destroy (protocols);
+  g_hash_table_unref (protocols);
 
   return retval;
 }
@@ -697,6 +700,13 @@ debug_window_get_name_owner_cb (TpDBusDaemon *proxy,
           COL_UNIQUE_NAME, out,
           -1);
 
+      if (priv->select_name != NULL &&
+          !tp_strdiff (name, priv->select_name))
+        {
+          gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->chooser), &iter);
+          tp_clear_pointer (&priv->select_name, g_free);
+        }
+
       g_free (name);
     }
 
@@ -1368,6 +1378,37 @@ tree_view_search_equal_func_cb (GtkTreeModel *model,
   return ret;
 }
 
+static void
+empathy_debug_window_select_name (EmpathyDebugWindow *self,
+    const gchar *name)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (self);
+  GtkTreeModel *model = GTK_TREE_MODEL (priv->service_store);
+  GtkTreeIter iter;
+  gchar *iter_name;
+  gboolean valid, found = FALSE;
+
+  for (valid = gtk_tree_model_get_iter_first (model, &iter);
+       valid;
+       valid = gtk_tree_model_iter_next (model, &iter))
+    {
+      gtk_tree_model_get (model, &iter,
+          COL_NAME, &iter_name,
+          -1);
+
+      if (!tp_strdiff (name, iter_name))
+        found = TRUE;
+
+      g_free (iter_name);
+
+      if (found)
+        break;
+    }
+
+  if (found)
+    gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->chooser), &iter);
+}
+
 static void
 am_prepared_cb (GObject *am,
     GAsyncResult *res,
@@ -1398,7 +1439,7 @@ am_prepared_cb (GObject *am,
   g_signal_connect (object, "key-press-event",
       G_CALLBACK (debug_window_key_press_event_cb), NULL);
 
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add (GTK_CONTAINER (object), vbox);
   gtk_widget_show (vbox);
 
@@ -1670,6 +1711,8 @@ debug_window_finalize (GObject *object)
   char *key;
   GList *values;
 
+  g_free (priv->select_name);
+
   g_hash_table_iter_init (&iter, priv->cache);
 
   while (g_hash_table_iter_next (&iter, (gpointer *) &key,
@@ -1678,7 +1721,7 @@ debug_window_finalize (GObject *object)
       debug_message_list_free (values);
     }
 
-  g_hash_table_destroy (priv->cache);
+  g_hash_table_unref (priv->cache);
 
   (G_OBJECT_CLASS (empathy_debug_window_parent_class)->finalize) (object);
 }
@@ -1700,6 +1743,9 @@ debug_window_dispose (GObject *object)
   if (priv->name_owner_changed_signal != NULL)
     tp_proxy_signal_connection_disconnect (priv->name_owner_changed_signal);
 
+  if (priv->new_debug_message_signal != NULL)
+    tp_proxy_signal_connection_disconnect (priv->new_debug_message_signal);
+
   if (priv->proxy != NULL)
     {
       debug_window_set_enabled (EMPATHY_DEBUG_WINDOW (object), FALSE);
@@ -1707,9 +1753,6 @@ debug_window_dispose (GObject *object)
       g_object_unref (priv->proxy);
     }
 
-  if (priv->new_debug_message_signal != NULL)
-    tp_proxy_signal_connection_disconnect (priv->new_debug_message_signal);
-
   if (priv->service_store != NULL)
     g_object_unref (priv->service_store);
 
@@ -1747,3 +1790,20 @@ empathy_debug_window_new (GtkWindow *parent)
   return GTK_WIDGET (g_object_new (EMPATHY_TYPE_DEBUG_WINDOW,
       "transient-for", parent, NULL));
 }
+
+void
+empathy_debug_window_show (EmpathyDebugWindow *self,
+    const gchar *name)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (self);
+
+  if (priv->service_store != NULL)
+    {
+      empathy_debug_window_select_name (self, name);
+    }
+  else
+    {
+      g_free (priv->select_name);
+      priv->select_name = g_strdup (name);
+    }
+}
