@@ -31,7 +31,6 @@
 
 #include <telepathy-yell/telepathy-yell.h>
 
-#include <libempathy/empathy-channel-factory.h>
 #include <libempathy/empathy-presence-manager.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
@@ -1116,12 +1115,11 @@ approve_channels (TpSimpleApprover *approver,
         event_manager_call_channel_got_contact_cb,
         approval, NULL, G_OBJECT (self));
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
+  else if (EMPATHY_IS_TP_FILE (channel))
     {
       TpHandle handle;
-      EmpathyTpFile *tp_file = empathy_tp_file_new (channel);
 
-      approval->handler_instance = G_OBJECT (tp_file);
+      approval->handler_instance = g_object_ref (channel);
 
       handle = tp_channel_get_handle (channel, NULL);
 
@@ -1130,10 +1128,31 @@ approve_channels (TpSimpleApprover *approver,
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
     {
-      event_manager_add (approval->manager, account, NULL, EMPATHY_EVENT_TYPE_AUTH,
-          GTK_STOCK_DIALOG_AUTHENTICATION, tp_account_get_display_name (account),
-          _("Password required"), approval,
-          event_manager_auth_process_func, NULL);
+      GHashTable *props;
+      const gchar * const *available_mechanisms;
+
+      props = tp_channel_borrow_immutable_properties (channel);
+      available_mechanisms = tp_asv_get_boxed (props,
+          TP_PROP_CHANNEL_INTERFACE_SASL_AUTHENTICATION_AVAILABLE_MECHANISMS,
+          G_TYPE_STRV);
+
+      if (tp_strv_contains (available_mechanisms, "X-TELEPATHY-PASSWORD"))
+        {
+          event_manager_add (approval->manager, account, NULL,
+              EMPATHY_EVENT_TYPE_AUTH,
+              GTK_STOCK_DIALOG_AUTHENTICATION,
+              tp_account_get_display_name (account),
+              _("Password required"), approval,
+              event_manager_auth_process_func, NULL);
+        }
+      else
+        {
+          GError error = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+              "Support only X-TELEPATHY-PASSWORD auth method" };
+
+          tp_add_dispatch_operation_context_fail (context, &error);
+          return;
+        }
     }
   else
     {
@@ -1341,7 +1360,7 @@ empathy_event_manager_class_init (EmpathyEventManagerClass *klass)
       G_SIGNAL_RUN_LAST,
       0,
       NULL, NULL,
-      g_cclosure_marshal_VOID__POINTER,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE,
       1, G_TYPE_POINTER);
 
@@ -1351,7 +1370,7 @@ empathy_event_manager_class_init (EmpathyEventManagerClass *klass)
       G_SIGNAL_RUN_LAST,
       0,
       NULL, NULL,
-      g_cclosure_marshal_VOID__POINTER,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, G_TYPE_POINTER);
 
   signals[EVENT_UPDATED] =
@@ -1360,7 +1379,7 @@ empathy_event_manager_class_init (EmpathyEventManagerClass *klass)
       G_SIGNAL_RUN_LAST,
       0,
       NULL, NULL,
-      g_cclosure_marshal_VOID__POINTER,
+      g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, G_TYPE_POINTER);
 
   g_type_class_add_private (object_class, sizeof (EmpathyEventManagerPriv));
@@ -1371,9 +1390,8 @@ empathy_event_manager_init (EmpathyEventManager *manager)
 {
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
-  TpDBusDaemon *dbus;
   GError *error = NULL;
-  EmpathyChannelFactory *factory;
+  TpAccountManager *am;
 
   manager->priv = priv;
 
@@ -1389,20 +1407,10 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   g_signal_connect (priv->contact_manager, "members-changed",
     G_CALLBACK (event_manager_members_changed_cb), manager);
 
-  dbus = tp_dbus_daemon_dup (&error);
-  if (dbus == NULL)
-    {
-      DEBUG ("Failed to get TpDBusDaemon: %s", error->message);
-      g_error_free (error);
-      return;
-    }
+   am = tp_account_manager_dup ();
 
-  priv->approver = tp_simple_approver_new (dbus, "Empathy.EventManager", FALSE,
-      approve_channels, manager, NULL);
-
-  /* EmpathyTpChat relies on this feature being prepared */
-  tp_base_client_add_connection_features_varargs (priv->approver,
-    TP_CONNECTION_FEATURE_CAPABILITIES, 0);
+  priv->approver = tp_simple_approver_new_with_am (am, "Empathy.EventManager",
+      FALSE, approve_channels, manager, NULL);
 
   /* Private text channels */
   tp_base_client_take_approver_filter (priv->approver,
@@ -1449,7 +1457,7 @@ empathy_event_manager_init (EmpathyEventManager *manager)
    * EmpathyTpChat and its users to not depend on the connection being
    * prepared with capabilities. I chose the former, obviously. :-) */
 
-  priv->auth_approver = tp_simple_approver_new (dbus,
+  priv->auth_approver = tp_simple_approver_new_with_am (am,
       "Empathy.AuthEventManager", FALSE, approve_channels, manager,
       NULL);
 
@@ -1463,11 +1471,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
           TP_IFACE_CHANNEL_INTERFACE_SASL_AUTHENTICATION,
         NULL));
 
-  factory = empathy_channel_factory_dup ();
-
-  tp_base_client_set_channel_factory (priv->approver,
-      TP_CLIENT_CHANNEL_FACTORY (factory));
-
   if (!tp_base_client_register (priv->approver, &error))
     {
       DEBUG ("Failed to register Approver: %s", error->message);
@@ -1480,8 +1483,7 @@ empathy_event_manager_init (EmpathyEventManager *manager)
       g_error_free (error);
     }
 
-  g_object_unref (factory);
-  g_object_unref (dbus);
+  g_object_unref (am);
 }
 
 EmpathyEventManager *
