@@ -47,8 +47,6 @@
 #include <libempathy/empathy-tp-contact-factory.h>
 
 #include <libempathy-gtk/empathy-contact-dialogs.h>
-#include <libempathy-gtk/empathy-contact-list-store.h>
-#include <libempathy-gtk/empathy-contact-list-view.h>
 #include <libempathy-gtk/empathy-live-search.h>
 #include <libempathy-gtk/empathy-contact-blocking-dialog.h>
 #include <libempathy-gtk/empathy-contact-search-dialog.h>
@@ -76,7 +74,6 @@
 #include "empathy-chatrooms-window.h"
 #include "empathy-event-manager.h"
 #include "empathy-ft-manager.h"
-#include "empathy-migrate-butterfly-logs.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
@@ -98,6 +95,11 @@ enum {
 	PAGE_NO_MATCH
 };
 
+enum {
+	PROP_0,
+	PROP_SHELL_RUNNING
+};
+
 G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
 
 #define GET_PRIV(self) ((EmpathyMainWindowPriv *)((EmpathyMainWindow *) self)->priv)
@@ -164,8 +166,7 @@ struct _EmpathyMainWindowPriv {
 	/* Actions that are enabled when there are connected accounts */
 	GList                  *actions_connected;
 
-	/* The idle event source to migrate butterfly's logs */
-	guint butterfly_log_migration_members_changed_id;
+	gboolean               shell_running;
 };
 
 static void
@@ -351,7 +352,7 @@ main_window_auth_display (EmpathyMainWindow *window,
 	GtkWidget *add_button;
 	GtkWidget *close_button;
 	GtkWidget *action_area;
-	GtkWidget *action_table;
+	GtkWidget *action_grid;
 	const gchar *icon_name;
 	gchar *str;
 
@@ -398,19 +399,15 @@ main_window_auth_display (EmpathyMainWindow *window,
 	gtk_widget_set_tooltip_text (close_button, _("Disconnect"));
 	gtk_widget_show (close_button);
 
-	action_table = gtk_table_new (1, 2, FALSE);
-	gtk_table_set_col_spacings (GTK_TABLE (action_table), 6);
-	gtk_widget_show (action_table);
+	action_grid = gtk_grid_new ();
+	gtk_grid_set_column_spacing (GTK_GRID (action_grid), 6);
+	gtk_widget_show (action_grid);
 
 	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
-	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (action_area), action_grid, FALSE, FALSE, 0);
 
-	gtk_table_attach (GTK_TABLE (action_table), add_button, 0, 1, 0, 1,
-			  (GtkAttachOptions) (GTK_SHRINK),
-			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
-	gtk_table_attach (GTK_TABLE (action_table), close_button, 1, 2, 0, 1,
-			  (GtkAttachOptions) (GTK_SHRINK),
-			  (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_grid_attach (GTK_GRID (action_grid), add_button, 0, 0, 1, 1);
+	gtk_grid_attach (GTK_GRID (action_grid), close_button, 1, 0, 1, 1);
 
 	g_object_set_data_full (G_OBJECT (info_bar),
 				"event", event, NULL);
@@ -552,8 +549,25 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 				&data);
 }
 
+static gboolean
+main_window_load_events_idle_cb (gpointer user_data)
+{
+	EmpathyMainWindow *window = user_data;
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GSList *l;
+
+	l = empathy_event_manager_get_events (priv->event_manager);
+	while (l) {
+		main_window_event_added_cb (priv->event_manager, l->data,
+				window);
+		l = l->next;
+	}
+
+	return FALSE;
+}
+
 static void
-main_window_row_activated_cb (EmpathyContactListView *view,
+main_window_row_activated_cb (EmpathyIndividualView *view,
 			      GtkTreePath            *path,
 			      GtkTreeViewColumn      *col,
 			      EmpathyMainWindow      *window)
@@ -646,6 +660,12 @@ main_window_row_inserted_cb (GtkTreeModel      *model,
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 				PAGE_CONTACT_LIST);
 		gtk_widget_grab_focus (GTK_WIDGET (priv->individual_view));
+
+		/* The store is being filled, it will be done after an idle cb.
+		 * So we can then get events. If we do that too soon, event's
+		 * contact is not yet in the store and it won't get marked as
+		 * having events. */
+		g_idle_add (main_window_load_events_idle_cb, window);
 	}
 }
 
@@ -708,6 +728,130 @@ main_window_error_close_clicked_cb (GtkButton         *button,
 	main_window_remove_error (window, account);
 }
 
+static void
+main_window_error_upgrade_sw_clicked_cb (GtkButton         *button,
+					 EmpathyMainWindow *window)
+{
+	TpAccount *account;
+	GtkWidget *dialog;
+
+	account = g_object_get_data (G_OBJECT (button), "account");
+	main_window_remove_error (window, account);
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+		GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+		GTK_BUTTONS_OK,
+		_("Sorry, %s accounts can’t be used until your %s software is updated."),
+		tp_account_get_protocol (account),
+		tp_account_get_protocol (account));
+
+	g_signal_connect_swapped (dialog, "response",
+		G_CALLBACK (gtk_widget_destroy),
+		dialog);
+
+	gtk_widget_show (dialog);
+}
+
+static void
+main_window_upgrade_software_error (EmpathyMainWindow *window,
+				    TpAccount         *account)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkWidget *info_bar;
+	GtkWidget *content_area;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *upgrade_button;
+	GtkWidget *close_button;
+	GtkWidget *action_area;
+	GtkWidget *action_grid;
+	gchar     *str;
+	const gchar     *icon_name;
+	const gchar *error_message;
+	gboolean user_requested;
+
+	error_message =
+		empathy_account_get_error_message (account, &user_requested);
+
+	if (user_requested) {
+		return;
+	}
+
+	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
+					       tp_account_get_display_name (account),
+					       error_message);
+
+	/* If there are other errors, remove them */
+	main_window_remove_error (window, account);
+
+	info_bar = gtk_info_bar_new ();
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_ERROR);
+
+	gtk_widget_set_no_show_all (info_bar, TRUE);
+	gtk_box_pack_start (GTK_BOX (priv->errors_vbox), info_bar, FALSE, TRUE, 0);
+	gtk_widget_show (info_bar);
+
+	icon_name = tp_account_get_icon_name (account);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+	gtk_widget_show (image);
+
+	label = gtk_label_new (str);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_show (label);
+	g_free (str);
+
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
+	upgrade_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (upgrade_button), image);
+	gtk_widget_set_tooltip_text (upgrade_button, _("Update software..."));
+	gtk_widget_show (upgrade_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
+	close_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (close_button), image);
+	gtk_widget_set_tooltip_text (close_button, _("Close"));
+	gtk_widget_show (close_button);
+
+	action_grid = gtk_grid_new ();
+	gtk_grid_set_column_spacing (GTK_GRID (action_grid), 2);
+	gtk_widget_show (action_grid);
+
+	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (action_area), action_grid, FALSE, FALSE, 0);
+
+	gtk_grid_attach (GTK_GRID (action_grid), upgrade_button, 0, 0, 1, 1);
+	gtk_grid_attach (GTK_GRID (action_grid), close_button, 1, 0, 1, 1);
+
+	g_object_set_data (G_OBJECT (info_bar), "label", label);
+	g_object_set_data_full (G_OBJECT (info_bar),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (upgrade_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (close_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+
+	g_signal_connect (upgrade_button, "clicked",
+			  G_CALLBACK (main_window_error_upgrade_sw_clicked_cb),
+			  window);
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (main_window_error_close_clicked_cb),
+			  window);
+
+	gtk_widget_set_tooltip_text (priv->errors_vbox, error_message);
+	gtk_widget_show (priv->errors_vbox);
+
+	g_hash_table_insert (priv->errors, g_object_ref (account), info_bar);
+}
+
 static void
 main_window_error_display (EmpathyMainWindow *window,
 			   TpAccount         *account)
@@ -721,12 +865,18 @@ main_window_error_display (EmpathyMainWindow *window,
 	GtkWidget *edit_button;
 	GtkWidget *close_button;
 	GtkWidget *action_area;
-	GtkWidget *action_table;
+	GtkWidget *action_grid;
 	gchar     *str;
 	const gchar     *icon_name;
 	const gchar *error_message;
 	gboolean user_requested;
 
+	if (!tp_strdiff (TP_ERROR_STR_SOFTWARE_UPGRADE_REQUIRED,
+			 tp_account_get_detailed_error (account, NULL))) {
+		main_window_upgrade_software_error (window, account);
+		return;
+	}
+
 	error_message =
 		empathy_account_get_error_message (account, &user_requested);
 
@@ -789,22 +939,16 @@ main_window_error_display (EmpathyMainWindow *window,
 	gtk_widget_set_tooltip_text (close_button, _("Close"));
 	gtk_widget_show (close_button);
 
-	action_table = gtk_table_new (1, 3, FALSE);
-	gtk_table_set_col_spacings (GTK_TABLE (action_table), 2);
-	gtk_widget_show (action_table);
+	action_grid = gtk_grid_new ();
+	gtk_grid_set_column_spacing (GTK_GRID (action_grid), 2);
+	gtk_widget_show (action_grid);
 
 	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
-	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (action_area), action_grid, FALSE, FALSE, 0);
 
-	gtk_table_attach (GTK_TABLE (action_table), retry_button, 0, 1, 0, 1,
-										(GtkAttachOptions) (GTK_SHRINK),
-										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
-	gtk_table_attach (GTK_TABLE (action_table), edit_button, 1, 2, 0, 1,
-										(GtkAttachOptions) (GTK_SHRINK),
-										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
-	gtk_table_attach (GTK_TABLE (action_table), close_button, 2, 3, 0, 1,
-										(GtkAttachOptions) (GTK_SHRINK),
-										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_grid_attach (GTK_GRID (action_grid), retry_button, 0, 0, 1, 1);
+	gtk_grid_attach (GTK_GRID (action_grid), edit_button, 1, 0, 1, 1);
+	gtk_grid_attach (GTK_GRID (action_grid), close_button, 2, 0, 1, 1);
 
 	g_object_set_data (G_OBJECT (info_bar), "label", label);
 	g_object_set_data_full (G_OBJECT (info_bar),
@@ -1021,7 +1165,7 @@ main_window_setup_balance_create_widget (EmpathyMainWindow *window,
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	GtkWidget *hbox, *image, *label, *button;
 
-	hbox = gtk_hbox_new (FALSE, 6);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
 	/* protocol icon */
 	image = gtk_image_new ();
@@ -1228,8 +1372,8 @@ empathy_main_window_finalize (GObject *window)
 	g_object_unref (priv->individual_store);
 	g_object_unref (priv->contact_manager);
 	g_object_unref (priv->sound_mgr);
-	g_hash_table_destroy (priv->errors);
-	g_hash_table_destroy (priv->auths);
+	g_hash_table_unref (priv->errors);
+	g_hash_table_unref (priv->auths);
 
 	/* disconnect all handlers of status-changed signal */
 	g_hash_table_iter_init (&iter, priv->status_changed_handlers);
@@ -1237,7 +1381,7 @@ empathy_main_window_finalize (GObject *window)
 		g_signal_handler_disconnect (TP_ACCOUNT (key),
 					     GPOINTER_TO_UINT (value));
 
-	g_hash_table_destroy (priv->status_changed_handlers);
+	g_hash_table_unref (priv->status_changed_handlers);
 
 	g_signal_handlers_disconnect_by_func (priv->event_manager,
 			  		      main_window_event_added_cb,
@@ -1273,7 +1417,7 @@ static void
 main_window_chat_quit_cb (GtkAction         *action,
 			  EmpathyMainWindow *window)
 {
-	gtk_main_quit ();
+	gtk_widget_destroy (GTK_WIDGET (window));
 }
 
 static void
@@ -1373,7 +1517,7 @@ main_window_view_sort_contacts_cb (GtkRadioAction    *action,
 				   EmpathyMainWindow *window)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
-	EmpathyContactListStoreSort value;
+	EmpathyIndividualStoreSort value;
 	GSList      *group;
 	GType        type;
 	GEnumClass  *enum_class;
@@ -1854,7 +1998,8 @@ empathy_main_window_show_preferences (EmpathyMainWindow *window,
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 
 	if (priv->preferences == NULL) {
-		priv->preferences = empathy_preferences_new (GTK_WINDOW (window));
+		priv->preferences = empathy_preferences_new (GTK_WINDOW (window),
+		                                             priv->shell_running);
 		g_object_add_weak_pointer (G_OBJECT (priv->preferences),
 					   (gpointer) &priv->preferences);
 
@@ -2043,25 +2188,17 @@ account_manager_prepared_cb (GObject      *source_object,
 	g_list_free (accounts);
 }
 
-static void
-main_window_members_changed_cb (EmpathyContactList *list,
-				EmpathyContact     *contact,
-				EmpathyContact     *actor,
-				guint               reason,
-				gchar              *message,
-				gboolean            is_member,
-				EmpathyMainWindow  *window)
+void
+empathy_main_window_set_shell_running (EmpathyMainWindow *window,
+                                       gboolean          shell_running)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 
-	if (!is_member)
+	if (priv->shell_running == shell_running)
 		return;
 
-	if (!empathy_migrate_butterfly_logs (contact)) {
-		g_signal_handler_disconnect (list,
-			priv->butterfly_log_migration_members_changed_id);
-		priv->butterfly_log_migration_members_changed_id = 0;
-	}
+	priv->shell_running = shell_running;
+	g_object_notify (G_OBJECT (window), "shell-running");
 }
 
 static GObject *
@@ -2082,14 +2219,65 @@ empathy_main_window_constructor (GType type,
 	return window;
 }
 
+static void
+empathy_main_window_set_property (GObject       *object,
+                                  guint         property_id,
+                                  const GValue *value,
+                                  GParamSpec    *pspec)
+{
+	EmpathyMainWindow *self = EMPATHY_MAIN_WINDOW (object);
+	EmpathyMainWindowPriv *priv = GET_PRIV (self);
+
+	switch (property_id)
+	{
+		case PROP_SHELL_RUNNING:
+			priv->shell_running = g_value_get_boolean (value);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
+static void
+empathy_main_window_get_property (GObject    *object,
+                                  guint      property_id,
+                                  GValue     *value,
+                                  GParamSpec *pspec)
+{
+	EmpathyMainWindow *self = EMPATHY_MAIN_WINDOW (object);
+	EmpathyMainWindowPriv *priv = GET_PRIV (self);
+
+	switch (property_id)
+	{
+		case PROP_SHELL_RUNNING:
+			g_value_set_boolean (value, priv->shell_running);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
 static void
 empathy_main_window_class_init (EmpathyMainWindowClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GParamSpec *pspec;
 
 	object_class->finalize = empathy_main_window_finalize;
 	object_class->constructor = empathy_main_window_constructor;
 
+	object_class->set_property = empathy_main_window_set_property;
+	object_class->get_property = empathy_main_window_get_property;
+
+	pspec = g_param_spec_boolean ("shell-running",
+	                              "Shell running",
+	                              "Whether the Shell is running or not",
+	                              FALSE,
+	                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+	g_object_class_install_property (object_class, PROP_SHELL_RUNNING, pspec);
+
 	g_type_class_add_private (object_class, sizeof (EmpathyMainWindowPriv));
 }
 
@@ -2105,7 +2293,6 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	GtkToolItem              *item;
 	gboolean                  show_offline;
 	gchar                    *filename;
-	GSList                   *l;
 	GtkTreeModel             *model;
 	GtkWidget                *search_vbox;
 	GtkWidget                *menubar;
@@ -2122,6 +2309,10 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gtk_window_set_role (GTK_WINDOW (window), "contact_list");
 	gtk_window_set_default_size (GTK_WINDOW (window), 225, 325);
 
+	/* don't finalize the widget on delete-event, just hide it */
+	g_signal_connect (window, "delete-event",
+		G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+
 	/* Set up interface */
 	filename = empathy_file_lookup ("empathy-main-window.ui", "src");
 	gui = empathy_builder_get_file (filename,
@@ -2282,10 +2473,6 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			EMPATHY_INDIVIDUAL_VIEW_FEATURE_ALL ^ EMPATHY_INDIVIDUAL_VIEW_FEATURE_PERSONA_DROP,
 			EMPATHY_INDIVIDUAL_FEATURE_ALL);
 
-	priv->butterfly_log_migration_members_changed_id = g_signal_connect (
-			priv->contact_manager, "members-changed",
-			G_CALLBACK (main_window_members_changed_cb), window);
-
 	gtk_widget_show (GTK_WIDGET (priv->individual_view));
 	gtk_container_add (GTK_CONTAINER (sw),
 			   GTK_WIDGET (priv->individual_view));
@@ -2346,13 +2533,6 @@ empathy_main_window_init (EmpathyMainWindow *window)
 			  G_CALLBACK (main_window_account_disabled_cb),
 			  window);
 
-	l = empathy_event_manager_get_events (priv->event_manager);
-	while (l) {
-		main_window_event_added_cb (priv->event_manager, l->data,
-				window);
-		l = l->next;
-	}
-
 	/* Show offline ? */
 	show_offline = g_settings_get_boolean (priv->gsettings_ui,
 					       EMPATHY_PREFS_UI_SHOW_OFFLINE);
